/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once

#include <dawn/dawn.h>
#include "../primitive.h"
#include "../texture.h"
#include "../tileset.h"
#include "../gui/font.h"
#include "../spritebatch.h"
#include "../shader.h"
#include "../../input/input.h"

/**
 * Creates a position debug tool.
 * 
 * @param debugPrimitive Primitive to debug
 * @param fontTexture Font Texture to use for debugging.
 * @param fontTileset Tileset for for font to use for debugging.
 * @return Pointer to debug renderer.
 */
positiondebug_t positionDebugCreate(
  primitive_t *debugPrimitive,
  texture_t *fontTexture, tileset_t *fontTileset
);

/**
 * Render a position debug tool.
 * 
 * @param shader Shader to use.
 * @param debug Debug tool to render.
 */
void positionDebugRender(shader_t *shader, positiondebug_t *debug);

/**
 * Dispose a previously created position debug tool.
 * 
 * @param debug Debug tool to dispose.
 */
void positionDebugDispose(positiondebug_t *debug);